"""Metadata templates for Dataverse."""

DATASET_JSON = """
{
  "datasetVersion": {
    "metadataBlocks": {
      "citation": {
        "fields": [
          {
            "value": "Youth in Austria 2005",
            "typeClass": "primitive",
            "multiple": false,
            "typeName": "title"
          },
          {
            "value": [
              {
                "authorName": {
                  "value": "LastAuthor1, FirstAuthor1",
                  "typeClass": "primitive",
                  "multiple": false,
                  "typeName": "authorName"
                },
                "authorAffiliation": {
                  "value": "AuthorAffiliation1",
                  "typeClass": "primitive",
                  "multiple": false,
                  "typeName": "authorAffiliation"
                }
              }
            ],
            "typeClass": "compound",
            "multiple": true,
            "typeName": "author"
          },
          {
            "value": [
              {
                "datasetContactEmail": {
                  "typeClass": "primitive",
                  "multiple": false,
                  "typeName": "datasetContactEmail",
                  "value": "ContactEmail1@mailinator.com"
                },
                "datasetContactName": {
                  "typeClass": "primitive",
                  "multiple": false,
                  "typeName": "datasetContactName",
                  "value": "LastContact1, FirstContact1"
                }
              }
            ],
            "typeClass": "compound",
            "multiple": true,
            "typeName": "datasetContact"
          },
          {
            "value": [
              {
                "dsDescriptionValue": {
                  "value": "DescriptionText",
                  "multiple": false,
                  "typeClass": "primitive",
                  "typeName": "dsDescriptionValue"
                }
              }
            ],
            "typeClass": "compound",
            "multiple": true,
            "typeName": "dsDescription"
          },
          {
            "value": [
              "Medicine, Health and Life Sciences"
            ],
            "typeClass": "controlledVocabulary",
            "multiple": true,
            "typeName": "subject"
          }
        ],
        "displayName": "Citation Metadata"
      }
    }
  }
}
"""



"""Utils used by Cli and Gui."""

import hashlib
from pathlib import Path


def create_unique_filename(local_dir: Path, filename: str):
    """Create a unique filename for a directory and original filename."""
    print(local_dir, filename)
    counter = 1
    local_path = local_dir / filename
    while local_path.exists():
        extension = filename.split(".")[-1]
        name = ".".join(filename.split(".")[:-1])
        print(name, extension)
        local_path = local_dir / (name + "_" + str(counter) + extension)
        counter += 1

    return local_path


def calculate_sha1_checksum(file_path):
    """Calculate the SHA-1 checksum of a file.

    Parameters
    ----------
    file_path:
        Path to the file.

    Returns
    -------
        SHA-1 checksum as a hexadecimal string.

    """
    sha1 = hashlib.sha1()
    try:
        with open(file_path, "rb") as f:
            for chunk in iter(lambda: f.read(8192), b""):
                sha1.update(chunk)
        return sha1.hexdigest()
    except FileNotFoundError:
        print(f"File not found: {file_path}")
        return None
    except IOError as e:
        print(f"I/O error: {e}")
        return None



"""QThread to transfer data."""

from pathlib import Path

import PySide6.QtCore
from ibridges import IrodsPath, Session, download

from ibridgescontrib.ibridgesdvn.dataverse import Dataverse
from ibridgescontrib.ibridgesdvn.dvn_operations import DvnOperations
from ibridgescontrib.ibridgesdvn.utils import calculate_sha1_checksum, create_unique_filename


class TransferDataThread(PySide6.QtCore.QThread):
    """Transfer data between local and iRODS."""

    result = PySide6.QtCore.Signal(dict)
    current_progress = PySide6.QtCore.Signal(list)

    # pylint: disable=too-many-positional-arguments
    def __init__(
        self,
        ienv_path: Path,
        logger,
        dvn_ops: DvnOperations,
        dvn_url: str,
        dvn_token: str,
        tempdir: Path,
        dataset_id: str,
        checksum: bool,
    ):
        """Pass parameters.

        ienv_path : Path
            path to the irods_environment.json to create a new session.
        logger : logging.Logger
            Logger
        irods_paths : list
            List of absolute iRODS paths to be transferred.
        """
        super().__init__()

        self.logger = logger
        self.thread_session = Session(irods_env=ienv_path)
        self.logger.debug("DATAVERSE: Transfer data thread: Created new session.")
        self.dvn_ops = dvn_ops
        self.checksum = checksum
        self.tempdir = tempdir
        self.dataset_id = dataset_id
        self.dvn_url = dvn_url
        self.irods_paths = [
            IrodsPath(self.thread_session, ip)
            for ip in self.dvn_ops.get_paths(self.dvn_url, self.dataset_id)
        ]
        self.dvn_api = Dataverse(self.dvn_url, dvn_token)

    def _delete_session(self):
        self.thread_session.close()
        del self.dvn_api
        if self.thread_session.irods_session is None:
            self.logger.debug(
                "DATAVERSE: Transfer data thread: Thread session successfully deleted."
            )
        else:
            self.logger.debug("DATAVERSE: Transfer data thread: Thread session still exists.")

    def run(self):
        """Run the thread."""
        file_count = 0
        file_failed = 0
        transfer_out = {}
        transfer_out["error"] = ""

        for irods_path in self.irods_paths:
            if irods_path.dataobject_exists():
                try:
                    local_path = create_unique_filename(self.tempdir, irods_path.name)
                    download(irods_path, local_path, overwrite=True)
                    self.logger.info(
                        "DATAVERSE: Download %s --> %s", str(irods_path), str(local_path)
                    )
                    self.dvn_api.add_datafile_to_dataset(self.dataset_id, local_path)
                    self.logger.info(
                        "DATAVERSE: Upload %s --> %s", str(local_path), self.dataset_id
                    )
                    # check checksums
                    if self.checksum:
                        sha1 = calculate_sha1_checksum(local_path)
                        dvn_sha1 = self.dvn_api.get_checksum_by_filename(
                            self.dataset_id, local_path.name
                        )
                        if sha1 != dvn_sha1:
                            self.logger.error(
                                "DATAVERSE: ERROR: transfer  %s --> %s failed, checksum error",
                                str(local_path),
                                self.dataset_id,
                            )
                            file_failed += 1
                            transfer_out["error"] = (
                                transfer_out["error"]
                                + f"\nTransfer failed, checksum error for {str(irods_path)}."
                            )
                        else:
                            self.logger.info(
                                "DATAVERSE: transfer  %s --> %s checksum ok",
                                str(local_path),
                                self.dataset_id,
                            )
                            file_count += 1
                    self.dvn_ops.rm_file(self.dvn_url, self.dataset_id, str(irods_path))
                    local_path.unlink()

                except Exception as err:  # pylint: disable=W0718
                    self.logger.error("DATAVERSE: Error in download and upload: %s", repr(err))
                    transfer_out["error"] = (
                        transfer_out["error"] + "\nSomething went wrong, check the logs."
                    )
            else:
                self.logger.error("DATAVERSE: ERROR: iRODS %s not found.", str(irods_path))
                file_failed += 1
                transfer_out["error"] = transfer_out["error"] + f"\n{irods_path} not found."
            self.current_progress.emit([file_count, len(self.irods_paths), file_failed])

        self._delete_session()
        self.result.emit(transfer_out)



"""Create links to python classes."""

from ibridgescontrib.ibridgesdvn.dvn_dataset_commands import (
    CliDvnAddFile,
    CliDvnCleanUp,
    CliDvnCreateDataset,
    CliDvnPush,
    CliDvnRmFile,
    CliDvnStatus,
)
from ibridgescontrib.ibridgesdvn.dvn_init import CliDvnInit, CliDvnSwitch
from ibridgescontrib.ibridgesdvn.dvn_setup import CliDvnAlias

SETUP_COMMANDS = [CliDvnAlias]
INIT_COMMANDS = [CliDvnInit]
SWITCH_COMMANDS = [CliDvnSwitch]
PUSH_COMMANDS = [CliDvnPush]
STATUS_COMMANDS = [CliDvnStatus]
CLEANUP_COMMANDS = [CliDvnCleanUp]
RM_FILE_COMMANDS = [CliDvnRmFile]
ADD_FILE_COMMANDS = [CliDvnAddFile]
CREATE_DS_COMMANDS = [CliDvnCreateDataset]

__all__ = ["get_dataverse_tab"]

# pylint: disable=import-outside-toplevel
def get_dataverse_tab():
    """Import for GUI."""
    try:
        from ibridgescontrib.ibridgesdvn.gui.gui_dataverse import (
            DataverseTab,
        )
    except ImportError as e:
        raise ImportError(
            "GUI dependencies missing. Install with:\n"
            "  pip install ibridgescontrib[gui]"
        ) from e
    return DataverseTab



"""Save operations in a file to be executed later."""

import json
import warnings
from pathlib import Path
from pprint import pprint

DVN_OPS_PATH = Path.home() / ".dvn" / "dvn_active_ops_log.json"


class DvnOperations:
    """Store dataverse operations which need to be executed."""

    def __init__(self, ops_log_path: Path = DVN_OPS_PATH):
        """Init with current operations in log file."""
        self.ops_log_path = ops_log_path
        self.ops = self._read_ops_log()
        if self.ops:
            self.validate_ops_format()

    def _read_ops_log(self):
        try:
            with open(self.ops_log_path, "r", encoding="utf-8") as handle:
                return json.load(handle)
        except FileNotFoundError:
            return {}

    def validate_ops_format(self):
        """Validate the format of the ops dictionary.

        {
            dataverse instance1:{
                "add_file": [{"dataset": "ds_id", "irods_paths": ["path1", "path2", ...]}]
            },
            dataverse instance2:{
                ...
            }
        }
        """
        for dataverse, ops in self.ops.items():
            if not isinstance(ops, dict):
                raise ValueError(f"Entry {dataverse} in {DVN_OPS_PATH} needs to be dictionary.")
            if "add_file" in ops:
                if not isinstance(ops["add_file"], list):
                    raise ValueError(f"Entry 'add_file' in {dataverse} needs to be list.")
                for item in ops["add_file"]:
                    if not isinstance(item, dict):
                        raise ValueError(f"Entry 'add_file' in {dataverse} needs to contain dicts.")

    def show(self):
        """Print operations to screen."""
        pprint(self.ops)

    def _get_dataset_ids(self, dataverse_url):
        return [item["dataset"] for item in self.ops[dataverse_url]["add_file"]]

    def _get_paths_for_dataset(self, dataset_id, items):
        for item in items:
            if item["dataset"] == dataset_id:
                if "irods_paths" in item:
                    return item["irods_paths"]
        return None

    def _add_path_to_dataset(self, dataset_id, new_path, items):
        for item in items:
            if item["dataset"] == dataset_id:
                if "irods_paths" not in item:
                    item["irods_paths"] = [new_path]
                else:
                    item["irods_paths"].append(new_path)
                break

    def _remove_path_from_dataset(self, dataset_id, path_to_remove, items):
        for item in items:
            if item["dataset"] == dataset_id:
                if path_to_remove in item.get("irods_paths", []):
                    item["irods_paths"].remove(path_to_remove)

    def add_file(self, dataverse_url: str, dataset_id: str, irods_path: str):
        """Add file to list."""
        print(dataset_id)
        if dataverse_url in self.ops:
            dv_entry = self.ops[dataverse_url]["add_file"]
            if dataset_id in self._get_dataset_ids(dataverse_url):
                registered_paths = self._get_paths_for_dataset(
                    dataset_id, self.ops[dataverse_url]["add_file"]
                )
                if irods_path not in registered_paths:
                    self._add_path_to_dataset(
                        dataset_id, irods_path, self.ops[dataverse_url]["add_file"]
                    )
            else:
                # dataset_id does not exist yet
                new_entry = {"dataset": dataset_id, "irods_paths": [irods_path]}
                dv_entry.append(new_entry)

        else:
            # url does not exist
            self.ops[dataverse_url] = {
                "add_file": [{"dataset": dataset_id, "irods_paths": [irods_path]}]
            }

        self.validate_ops_format()
        self.save()

    def rm_file(self, dataverse_url: str, dataset_id: str, irods_path: str):
        """Remove file from the operations."""
        if dataverse_url not in self.ops:
            warnings.warn(f"No operations found for {dataverse_url}. Nothing to do.. Exit.")
            return
        if dataset_id not in self._get_dataset_ids(dataverse_url):
            warnings.warn(f"No operations found for {dataset_id}. Nothing to do.. Exit.")
            return
        registered_paths = self._get_paths_for_dataset(
            dataset_id, self.ops[dataverse_url]["add_file"]
        )

        if irods_path in registered_paths:
            self._remove_path_from_dataset(
                dataset_id, irods_path, self.ops[dataverse_url]["add_file"]
            )
        self.validate_ops_format()
        self.save()

    def get_paths(self, dataverse_url: str, dataset_id: str):
        """Return  irods_paths for a dataset."""
        if dataverse_url not in self.ops:
            return None
        if "add_file" not in self.ops[dataverse_url]:
            return None

        return self._get_paths_for_dataset(dataset_id, self.ops[dataverse_url]["add_file"])

    def save(self):
        """Save the operations object to json."""
        Path(self.ops_log_path).parent.mkdir(exist_ok=True, parents=True)
        with open(self.ops_log_path, "w", encoding="utf-8") as handle:
            json.dump(self.ops, handle, indent=4)

    def clean_up_datasets(self):
        """Remove all datasets entries with an empty irods_paths list."""
        for url in self.ops:
            dv_entry = self.ops[url]["add_file"]
            cleaned_entry = [entry for entry in dv_entry if entry["irods_paths"]]
            self.ops[url]["add_file"] = cleaned_entry
        self.save()



"""Example tab class to extend the iBridgesGUI app."""

import logging
import shutil
from pathlib import Path

import PySide6.QtWidgets
from ibridges import IrodsPath
from ibridges.session import Session
from ibridgesgui.config import get_last_ienv_path
from ibridgesgui.gui_utils import populate_table
from ibridgesgui.irods_tree_model import IrodsTreeModel

from ibridgescontrib.ibridgesdvn.dataverse import Dataverse
from ibridgescontrib.ibridgesdvn.dvn_config import DVNConf
from ibridgescontrib.ibridgesdvn.dvn_operations import DvnOperations
from ibridgescontrib.ibridgesdvn.gui.gui_popup_widgets import CreateDataset, CreateDvnURL
from ibridgescontrib.ibridgesdvn.gui.gui_thread import TransferDataThread
from ibridgescontrib.ibridgesdvn.gui.uiDataverse import Ui_Form

# pylint: disable=R0902


class DataverseTab(PySide6.QtWidgets.QWidget, Ui_Form):
    """Example tab for the iBridges GUI."""

    name = "Dataverse"

    def __init__(self, session: Session, app_name: str, logger: logging.Logger):
        """Initialize the example tab."""
        super().__init__()
        super().setupUi(self)
        self.logger = logger
        self.logger.info("Init third party tab: %s", self.name)

        self.session = session
        self.app_name = app_name

        self.dvn_conf = DVNConf(None)
        self.dvn_api = None
        self.dvn_ops = DvnOperations()
        self.url = None
        self.irods_model = None
        self.transfer_thread = None
        self.temp_dir = Path.home() / ".dvn" / "data"
        self.init_tab()

    def init_tab(self):
        """Initialise the widgets in the tab."""
        # add Dataverse URLs from config
        self.load_dataverse_conf()
        self.dv_url_select_box.currentTextChanged.connect(self._connect_to_dataverse)

        self.add_url_button.clicked.connect(self.add_dv_url)
        self.delete_url_button.clicked.connect(self.delete_dv_url)
        # self.dv_ds_edit --> get dataset id
        self.dv_ds_edit.textChanged.connect(self.dataset_edit_action)
        self.dv_create_ds_button.clicked.connect(self.dv_create_ds)
        # self.selected_data_table --> populate
        self.delete_selected_button.clicked.connect(self.dv_rm_file)
        self.dv_push_button.clicked.connect(self.dv_push)
        self.add_selected_button.clicked.connect(self.dv_add_file)
        self.add_selected_button.setEnabled(False)
        # self.irods_tree_view --> load collections
        self._init_irods_tree()

    def load_dataverse_conf(self):
        """Load or refresh drop down menu for configs."""
        items = [
            key for key in self.dvn_conf.dvns.keys() if self.dvn_conf.dvns[key].get("token", None)
        ]
        self.dv_url_select_box.clear()
        self.dv_url_select_box.addItems(items)
        if self.dvn_conf.cur_dvn is not None and self.dvn_conf.cur_dvn in items:
            self.dv_url_select_box.setCurrentIndex(items.index(self.dvn_conf.cur_dvn))
        else:
            self.dv_url_select_box.setCurrentIndex(0)
        self._connect_to_dataverse()

    def _connect_to_dataverse(self):
        self.error_label.clear()
        cur_url = self.dv_url_select_box.currentText()
        if cur_url == "":
            self.error_label.setText("Please create a Dataverse configuration.")
            return
        self.dvn_conf.set_dvn(cur_url)
        url, entry = self.dvn_conf.get_entry(cur_url)
        try:
            self.dvn_api = Dataverse(url, entry["token"])
            self.url = url
            self.dv_ds_edit.clear()
        except Exception as err:  # pylint: disable=W0718
            self.error_label.setText(
                f"Could not connect to {url} with {entry[1]['token']}, {repr(err)}"
            )

    def add_dv_url(self):
        """Add a new Dataverse URL with parameters."""
        self.error_label.clear()
        url_widget = CreateDvnURL(self.dvn_conf)
        url_widget.exec()
        self.load_dataverse_conf()

    def delete_dv_url(self):
        """Remove a URL from the configurations."""
        cur_url = self.dv_url_select_box.currentText()
        self.dvn_conf.delete_alias(cur_url)
        self.load_dataverse_conf()

    def dv_create_ds(self):
        """Create a dataset."""
        self.error_label.clear()
        if not self.dvn_api:
            self.error_label.setText(
                f"No API connection for {self.dv_url_select_box.currentText()}"
            )
            return
        self.error_label.clear()
        url_widget = CreateDataset(self.dvn_api, self.dv_ds_edit)
        url_widget.exec()
        doi = self.dv_ds_edit.text()
        if doi != "":
            self.logger.info("DATAVERSE: Created Dataset %s", doi)

    def dv_rm_file(self):
        """Remove a file from the table of selected files."""
        self.error_label.clear()
        dataset_id = self.dv_ds_edit.text()
        rows = set(idx.row() for idx in self.selected_data_table.selectedIndexes())
        for row in rows:
            path = self.selected_data_table.item(row, 0).text()
            self.dvn_ops.rm_file(self.url, dataset_id, path)
        self._populate_selected_data_table()

    def dv_push(self):
        """Download all objects in the table and upload to Dataverse dataset."""
        self.error_label.clear()
        dataset_id = self.dv_ds_edit.text()
        if dataset_id == "":
            self.error_label.setText("Select a dataset and refresh table.")
            return
        if not self.dvn_api.dataset_exists(dataset_id):
            self.error_label.setText("Dataset does not exist (any longer).")
            return
        if self.selected_data_table.rowCount() == 0:
            self.error_label.setText("Please add some data from the iRODS tree.")
            return

        self.temp_dir.mkdir(exist_ok=True)
        _, entry = self.dvn_conf.get_entry(self.url)

        try:
            self.transfer_thread = TransferDataThread(
                Path(get_last_ienv_path()), self.logger, self.dvn_ops, self.url, entry["token"],
                self.temp_dir, dataset_id, self.check_checksum_box.isChecked()
            )
        except Exception as err:
            self.error_label.setText(
                    f"Could not instantiate a new session from{get_last_ienv_path()}: {repr(err)}")
            self.logger.error("DATAVERSE: Could not instantiate a new session from %s: %s",
                              get_last_ienv_path(), repr(err))
            return

        self.transfer_thread.current_progress.connect(self._transfer_status)
        self.transfer_thread.result.connect(self._transfer_end)
        self.transfer_thread.finished.connect(self._finish_transfer_data)
        self.setCursor(PySide6.QtGui.QCursor(PySide6.QtCore.Qt.CursorShape.WaitCursor))
        self._enable_buttons(False)
        self.progress_bar.setValue(0)
        self.progress_bar.setMaximum(self.selected_data_table.rowCount())
        self.transfer_thread.start()

    def _enable_buttons(self, enable: bool):
        self.add_selected_button.setEnabled(enable)
        self.check_checksum_box.setEnabled(enable)
        self.delete_selected_button.setEnabled(enable)
        self.dv_ds_edit.setEnabled(enable)
        self.dv_create_ds_button.setEnabled(enable)
        self.dv_push_button.setEnabled(enable)
        self.selected_data_table.setEnabled(enable)
        self.dv_url_select_box.setEnabled(enable)
        self.add_url_button.setEnabled(enable)
        self.delete_url_button.setEnabled(enable)
        self.irods_tree_view.setEnabled(enable)

    def _transfer_status(self, state: list):
        obj_count, num_objs, obj_failed = state
        self.progress_bar.setValue(obj_count)
        text = f"{obj_count} of {num_objs} files; failed: {obj_failed}."
        self.status_label.setText(text)

    def _transfer_end(self, thread_output: dict):
        if thread_output["error"] != "":
            self.error_label.setText(thread_output["error"])
            self._populate_selected_data_table()
            shutil.rmtree(self.temp_dir)
            return

        shutil.rmtree(self.temp_dir)
        self._populate_selected_data_table()
        self.status_label.clear()
        self.error_label.setText(
                "Transfer finished, visit Dataverse and finish your publication.")
        self._enable_buttons(True)

    def _finish_transfer_data(self):
        self._enable_buttons(True)
        self.setCursor(PySide6.QtGui.QCursor(PySide6.QtCore.Qt.CursorShape.ArrowCursor))
        if self.transfer_thread:
            del self.transfer_thread

    def dv_add_file(self):
        """Add file from irods tree to table with selected files."""
        self.error_label.clear()
        # Retrieve irods paths
        irods_selection = self.irods_tree_view.selectedIndexes()
        if len(irods_selection) == 0:
            self.error_label.setText("Please select a data object.")
            return

        self.setCursor(PySide6.QtGui.QCursor(PySide6.QtCore.Qt.CursorShape.WaitCursor))
        for idx in irods_selection:
            irods_path = self.irods_model.irods_path_from_tree_index(idx)
            if irods_path.dataobject_exists():
                if irods_path.size > 9 * 10**9:
                    self.error_label.setText(
                            f"{irods_path} too large: size {irods_path.size} > {9 * 10**9}")
                else:
                    self.dvn_ops.add_file(self.url, self.dv_ds_edit.text(), str(irods_path))
            else:
                print("Not a data object")
                self.error_label.setText("Please only select data objects.")
        self._populate_selected_data_table()
        self.irods_tree_view.clearSelection()
        self.setCursor(PySide6.QtGui.QCursor(PySide6.QtCore.Qt.CursorShape.ArrowCursor))

    def _irods_root(self):
        """Retrieve lowest visible level in the iRODS tree for the user."""
        lowest = IrodsPath(self.session).absolute()
        while lowest.parent.exists() and str(lowest) != "/":
            lowest = lowest.parent
        return lowest

    def _init_irods_tree(self):
        root = self._irods_root()
        self.irods_model = IrodsTreeModel(self.irods_tree_view, root)
        self.irods_tree_view.setModel(self.irods_model)
        # self.irods_tree_view.headerItem().setText(0, "")
        self.irods_tree_view.expanded.connect(self.irods_model.refresh_subtree)
        self.irods_model.init_tree()
        # self.irods_tree_view.setCurrentIndex(self.irods_model.index(0, 0))
        self.irods_tree_view.expand(self.irods_model.index(0, 0))

        # hide unnecessary information
        self.irods_tree_view.setColumnHidden(1, True)
        self.irods_tree_view.setColumnHidden(2, True)
        self.irods_tree_view.setColumnHidden(3, True)
        self.irods_tree_view.setColumnHidden(4, True)
        self.irods_tree_view.setColumnHidden(5, True)

    def dataset_edit_action(self):
        """Load table upon dataset identifier."""
        self.error_label.clear()
        self._populate_selected_data_table()

    def _populate_selected_data_table(self):
        """Load irods path and size into table."""
        self.add_selected_button.setEnabled(True)
        self.selected_data_table.setRowCount(0)
        dataset_id = self.dv_ds_edit.text()
        # gather data from current stored files
        if dataset_id != "" and self.dvn_api.dataset_exists(dataset_id):
            if self.dvn_ops.get_paths(self.url, dataset_id):
                current_paths = [
                    (IrodsPath(self.session, path), IrodsPath(self.session, path).size)
                    for path in self.dvn_ops.get_paths(self.url, dataset_id)
                ]
                populate_table(self.selected_data_table, len(current_paths), current_paths)
        else:
            self.error_label.setText("Enter a valid Dataset Idenitifier.")



"""Main plugin file to show info."""

import argparse

from ibridges.cli.base import BaseCliCommand

from ibridgescontrib.ibridgesdvn.dvn_config import DVNConf, show_available


class CliDvnAlias(BaseCliCommand):
    """Subcommand to get information from the server."""

    names = ["dv-setup"]
    description = "Print existing Dataverse configurations or create new ones."
    examples = ["some_alias https://demo.dataverse.nl", "other_alias --delete"]

    @classmethod
    def _mod_parser(cls, parser):
        parser.add_argument(
            "alias",
            help="The new alias to be created",
            type=str,
            default=None,
            nargs="?",
        )
        parser.add_argument(
            "url", help="URL to the dataverse instance", type=str, default=None, nargs="?"
        )
        parser.add_argument(
            "--delete",
            "-d",
            help="Delete the alias.",
            action="store_true",
        )
        return parser

    @staticmethod
    def run_shell(session, parser, args):
        """Run alias command not available in the shell."""
        raise NotImplementedError()

    @classmethod
    def run_command(cls, args):
        """Create and manage aliases in the CLI."""
        parser = cls.get_parser(argparse.ArgumentParser)
        dvn_conf = DVNConf(parser)

        # Show available and selected aliases.
        if args.alias is None:
            show_available(dvn_conf)
            return

        # Delete alias
        if args.delete:
            dvn_conf.delete_alias(args.alias)
            return

        if args.url is None:
            parser.error("Supply the URL to the Dataverse server to set the alias.")

        if not dvn_conf.is_valid_url(args.url):
            parser.error(f"Supplied URL '{args.url}' is not a valid URL.")

        dvn_conf.set_alias(args.alias, args.url)
        show_available(dvn_conf)
